@@ -3,6 +3,7 @@
 from sqlalchemy import Boolean
 from sqlalchemy import CheckConstraint
 from sqlalchemy import Column
+from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import ForeignKey
 from sqlalchemy import Index
@@ -23,6 +24,7 @@ from alembic.testing import assert_raises_message
 from alembic.testing import combinations
 from alembic.testing import config
 from alembic.testing import eq_
+from alembic.testing import expect_warnings
 from alembic.testing import is_not_
 from alembic.testing import mock
 from alembic.testing.fixtures import op_fixture
@@ -409,31 +411,59 @@ class OpTest(TestBase):
             existing_server_default=esd(),
         )
 
-    def test_alter_column_schema_type_unnamed(self):
+    @combinations((True,), (False,), (None,), argnames="existing_nullable")
+    def test_alter_column_schema_type_unnamed(self, existing_nullable):
         context = op_fixture("mssql", native_boolean=False)
-        op.alter_column("t", "c", type_=Boolean(create_constraint=True))
-        context.assert_(
-            "ALTER TABLE t ALTER COLUMN c BIT",
-            "ALTER TABLE t ADD CHECK (c IN (0, 1))",
-        )
+        if existing_nullable is None:
+            with expect_warnings(
+                "MS-SQL ALTER COLUMN operations that specify type_= should"
+            ):
+                op.alter_column(
+                    "t",
+                    "c",
+                    type_=Boolean(create_constraint=True),
+                )
+            context.assert_(
+                "ALTER TABLE t ALTER COLUMN c BIT",
+                "ALTER TABLE t ADD CHECK (c IN (0, 1))",
+            )
+        else:
+            op.alter_column(
+                "t",
+                "c",
+                type_=Boolean(create_constraint=True),
+                existing_nullable=existing_nullable,
+            )
+            context.assert_(
+                f"ALTER TABLE t ALTER COLUMN c BIT "
+                f"{'NULL' if existing_nullable else 'NOT NULL'}",
+                "ALTER TABLE t ADD CHECK (c IN (0, 1))",
+            )
 
     def test_alter_column_schema_schema_type_unnamed(self):
         context = op_fixture("mssql", native_boolean=False)
         op.alter_column(
-            "t", "c", type_=Boolean(create_constraint=True), schema="foo"
+            "t",
+            "c",
+            type_=Boolean(create_constraint=True),
+            existing_nullable=False,
+            schema="foo",
         )
         context.assert_(
-            "ALTER TABLE foo.t ALTER COLUMN c BIT",
+            "ALTER TABLE foo.t ALTER COLUMN c BIT NOT NULL",
             "ALTER TABLE foo.t ADD CHECK (c IN (0, 1))",
         )
 
     def test_alter_column_schema_type_named(self):
         context = op_fixture("mssql", native_boolean=False)
         op.alter_column(
-            "t", "c", type_=Boolean(name="xyz", create_constraint=True)
+            "t",
+            "c",
+            type_=Boolean(name="xyz", create_constraint=True),
+            existing_nullable=False,
         )
         context.assert_(
-            "ALTER TABLE t ALTER COLUMN c BIT",
+            "ALTER TABLE t ALTER COLUMN c BIT NOT NULL",
             "ALTER TABLE t ADD CONSTRAINT xyz CHECK (c IN (0, 1))",
         )
 
@@ -443,10 +473,11 @@ class OpTest(TestBase):
             "t",
             "c",
             type_=Boolean(name="xyz", create_constraint=True),
+            existing_nullable=False,
             schema="foo",
         )
         context.assert_(
-            "ALTER TABLE foo.t ALTER COLUMN c BIT",
+            "ALTER TABLE foo.t ALTER COLUMN c BIT NOT NULL",
             "ALTER TABLE foo.t ADD CONSTRAINT xyz CHECK (c IN (0, 1))",
         )
 
@@ -482,10 +513,11 @@ class OpTest(TestBase):
             "c",
             type_=String(10),
             existing_type=Boolean(name="xyz", create_constraint=True),
+            existing_nullable=False,
         )
         context.assert_(
             "ALTER TABLE t DROP CONSTRAINT xyz",
-            "ALTER TABLE t ALTER COLUMN c VARCHAR(10)",
+            "ALTER TABLE t ALTER COLUMN c VARCHAR(10) NOT NULL",
         )
 
     def test_alter_column_schema_schema_type_existing_type(self):
@@ -495,11 +527,12 @@ class OpTest(TestBase):
             "c",
             type_=String(10),
             existing_type=Boolean(name="xyz", create_constraint=True),
+            existing_nullable=False,
             schema="foo",
         )
         context.assert_(
             "ALTER TABLE foo.t DROP CONSTRAINT xyz",
-            "ALTER TABLE foo.t ALTER COLUMN c VARCHAR(10)",
+            "ALTER TABLE foo.t ALTER COLUMN c VARCHAR(10) NOT NULL",
         )
 
     def test_alter_column_schema_type_existing_type_no_const(self):
@@ -850,6 +883,27 @@ class OpTest(TestBase):
         ck = [c for c in t1.constraints if isinstance(c, CheckConstraint)]
         eq_(ck[0].name, "ck_1")
 
+    def test_create_table_with_check_constraint_with_expr(self):
+        context = op_fixture()
+        foo_id = Column("foo_id", Integer)
+        t1 = op.create_table(
+            "some_table",
+            Column("id", Integer, primary_key=True),
+            foo_id,
+            CheckConstraint(foo_id > 5, name="ck_1"),
+        )
+        context.assert_(
+            "CREATE TABLE some_table ("
+            "id INTEGER NOT NULL, "
+            "foo_id INTEGER, "
+            "PRIMARY KEY (id), "
+            "CONSTRAINT ck_1 CHECK (foo_id > 5))"
+        )
+
+        ck = [c for c in t1.constraints if isinstance(c, CheckConstraint)]
+        eq_(ck[0].name, "ck_1")
+        eq_(len(ck), 1)
+
     def test_create_table_unique_constraint(self):
         context = op_fixture()
         t1 = op.create_table(
@@ -1034,6 +1088,58 @@ class OpTest(TestBase):
 
         context.assert_("COMMENT ON TABLE some_table IS NULL")
 
+    def test_create_table_event(self):
+        context = op_fixture()
+
+        events_triggered = []
+
+        TestTable = Table(
+            "tb_test", MetaData(), Column("c1", Integer, nullable=False)
+        )
+
+        @event.listens_for(Table, "before_create")
+        def record_before_event(table, conn, **kwargs):
+            events_triggered.append(("before_create", table.name))
+
+        @event.listens_for(Table, "after_create")
+        def record_after_event(table, conn, **kwargs):
+            events_triggered.append(("after_create", table.name))
+
+        op.create_table(TestTable)
+        op.drop_table(TestTable)
+        context.assert_("CREATE TABLE tb_test ()", "DROP TABLE tb_test")
+
+        assert events_triggered == [
+            ("before_create", "tb_test"),
+            ("after_create", "tb_test"),
+        ]
+
+    def test_drop_table_event(self):
+        context = op_fixture()
+
+        events_triggered = []
+
+        TestTable = Table(
+            "tb_test", MetaData(), Column("c1", Integer, nullable=False)
+        )
+
+        @event.listens_for(Table, "before_drop")
+        def record_before_event(table, conn, **kwargs):
+            events_triggered.append(("before_drop", table.name))
+
+        @event.listens_for(Table, "after_drop")
+        def record_after_event(table, conn, **kwargs):
+            events_triggered.append(("after_drop", table.name))
+
+        op.create_table(TestTable)
+        op.drop_table(TestTable)
+        context.assert_("CREATE TABLE tb_test ()", "DROP TABLE tb_test")
+
+        assert events_triggered == [
+            ("before_drop", "tb_test"),
+            ("after_drop", "tb_test"),
+        ]
+
 
 class SQLModeOpTest(TestBase):
     def test_auto_literals(self):
