@@ -6,6 +6,7 @@ import re
 import textwrap
 
 import sqlalchemy as sa
+from sqlalchemy import pool
 
 from alembic import command
 from alembic import testing
@@ -116,18 +117,9 @@ class PatchEnvironment:
 
 
 @testing.combinations(
-    (
-        False,
-        True,
-    ),
-    (
-        True,
-        False,
-    ),
-    (
-        True,
-        True,
-    ),
+    (False, True),
+    (True, False),
+    (True, True),
     argnames="transactional_ddl,transaction_per_migration",
     id_="rr",
 )
@@ -141,7 +133,9 @@ class ApplyVersionsFunctionalTest(PatchEnvironment, TestBase):
     branched_connection = False
 
     def setUp(self):
-        self.bind = _sqlite_file_db(future=self.future)
+        self.bind = _sqlite_file_db(
+            future=self.future, poolclass=pool.NullPool
+        )
         self.env = staging_env(sourceless=self.sourceless)
         self.cfg = _sqlite_testing_config(
             sourceless=self.sourceless, future=self.future
@@ -242,44 +236,51 @@ class ApplyVersionsFunctionalTest(PatchEnvironment, TestBase):
     def _test_002_upgrade(self):
         command.upgrade(self.cfg, self.c)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert db.dialect.has_table(db.connect(), "bat")
+
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert db.dialect.has_table(conn, "bat")
 
     def _test_003_downgrade(self):
         command.downgrade(self.cfg, self.a)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert not db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert not db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_004_downgrade(self):
         command.downgrade(self.cfg, "base")
         db = self.bind
-        assert not db.dialect.has_table(db.connect(), "foo")
-        assert not db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert not db.dialect.has_table(conn, "foo")
+            assert not db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_005_upgrade(self):
         command.upgrade(self.cfg, self.b)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_006_upgrade_again(self):
         command.upgrade(self.cfg, self.b)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
     def _test_007_stamp_upgrade(self):
         command.stamp(self.cfg, self.c)
         db = self.bind
-        assert db.dialect.has_table(db.connect(), "foo")
-        assert db.dialect.has_table(db.connect(), "bar")
-        assert not db.dialect.has_table(db.connect(), "bat")
+        with db.connect() as conn:
+            assert db.dialect.has_table(conn, "foo")
+            assert db.dialect.has_table(conn, "bar")
+            assert not db.dialect.has_table(conn, "bat")
 
 
 class LegacyApplyVersionsFunctionalTest(ApplyVersionsFunctionalTest):
@@ -795,7 +796,7 @@ class IgnoreFilesTest(TestBase):
     sourceless = False
 
     def setUp(self):
-        self.bind = _sqlite_file_db()
+        self.bind = _sqlite_file_db(poolclass=pool.NullPool)
         self.env = staging_env(sourceless=self.sourceless)
         self.cfg = _sqlite_testing_config(sourceless=self.sourceless)
 
